function [JSRad,diagnosticJSR,sequenceJSR]=JSR_v14(A,MaxNumIter,precisionJSR,UseNorm, ...
    StartingStep,norme,MaxIterSDP,OptimizeEvery,maxNumOptimization, ...
    askQuestion,precisionNorm,NumPause,cleanWorkspace,epsilon, ...
    PruningEvery,MaxBranchesAlive,SpecialCases,ConsiderNewSMP)

%
% JSR     Joint Spectral Radius evaluation for a family of matrices.
%
% function [JSRad,diagnosticJSR]=JSR_v14(A,MaxNumIter,precisionJSR,UseNorm,
%           StartingStep,norme,MaxIterSDP,OptimizeEvery,maxNumOptimization,
%           askQuestion,precisionNorm,NumPause,cleanWorkspace,epsilon,
%           PruningEvery,MaxBranchesAlive,SpecialCases,ConsiderNewSMP)
%
% Input:
%
%                    - Compulsory -
%
%   A              Initial matrix family.
%                   Ex. A(:,:,1)=[1 1;0 1]; A(:,:,2)=A(:,:,1)'.
%   MaxNumIter     Max number of iteration for the algorithm, i.e. max
%                   length of the product the algorithm will construct.
%   precisionJSR   Max difference required in between upper and lower bound
%                   for the estimate of the Joint Spectral Radius
%   UseNorm        It is possible to choose the type of Norm to be used
%                   during steps>= StartingStep. Select:
%                   'E' for Ellipsoidal Norms
%                   'G' for Gripenberg method
%
%                     - Optional -
%
%   StartingStep   Step in which the algorithm starts to cut branches.
%                   Choose a value >=2. Default value
%                   max(floor(log(size of matrices)/log(number of
%                   matrices)),2)
%   norme          Define the matrix norms you want to use.
%                   Default value [1,2,Inf]
%   MaxIterSDP     Max number of iteration for the Semidefinite
%                   Programming. Default value 2
%   OptimizeEvery  Define how many steps wait before optimize again.
%                   Default value setted equal to 'StartingStep'
%   maxNumOptimization
%                  Allow to Define how many times the algorithm will
%                   optimize. Default value 3.
%   askQuestion    Vector of two elements. Set the first element to 0 if
%                   you want to know the exact time of an evaluation,
%                   the algorithm will not ask any question, choose 1
%                   otherwise. Set the second element to
%                   0 if you do not want the algorithm to display
%                   intermidiate results. Default value [0,1].
%   precisionNorm  Max difference required in between upper and lower bound
%                   of the value erre, estimate for the Joint Spectral Radius
%                   via Semidefinite Programming. Default value=precisionJSR
%   NumPause       Specify a value different from 0 if you want to pause
%                   the algorithm. For instance choose 4 if you want the
%                   algorithm to pause every 4 loop, 0 otherwise. If
%                   'askQuestion(1)=1' the algorithm will ask every
%                   NumPause what to do, otherwise just considering value
%                   'MaxBranchesAlive' automatically stop if the number of
%                   'branchesAlive' at step i becames bigger than value
%                   'MaxBranchesAlive'. Default value is 1.
%   cleanWorkspace Choose 1 if you want to erase all the global variables
%                   at the end of the algorithm, 0 otherwise.
%                   Global variables if not erased can be reactived at the
%                   end of an algorithm simply calling the command
%                   "global name_of_the_variable" in the command line.
%                   Consider also that this algorithm every time it is
%                   started cleans these variables to avoid
%                   superimpositions. Default value 0
%   epsilon        Value used in RectNorm_v07, default value 10^-9
%
%   PruningEvery   Choose 1 if you want to prune dry branches after
%                   optimization every step>=StartingStep.
%                   Choose 0 if you do not want to prune at all.
%                   Choose 2, 3... if you want the algorithm to prune
%                   every 2, 3 steps.
%                   Pruning dry branches allows to analyze long products.
%                   Default value 1
% MaxBranchesAlive Maximum number of Branches alive allowed (see NumPause)
%                   Default value 1000 if 'UseNorm' is not
%                    'G', 4000 otherwise.
%   SpecialCases   Choose 1 if you want the algorithm to consider known
%                   special cases, 0 otherwise. Default value 1.
%   ConsiderNewSMP Choose 1 if you want to optimize every time the
%                   algorithm find a new candidate SMP in a step greater
%                   than StartingStep, 0 if you do not want. Consider that
%                   also these optimizations are counted in the evaluation
%                   of 'maxNumOptimization'. Default value 0
% Default values:
% [JSRad,diagnosticJSR,sequenceJSR]=JSR_v14(A,MaxNumIter,precisionJSR,UseNorm, ...
%     StartingStep,norme,MaxIterSDP,OptimizeEvery,maxNumOptimization, ...
%     askQuestion,precisionNorm,NumPause,cleanWorkspace,epsilon, ...
%     PruningEvery,MaxBranchesAlive,SpecialCases,ConsiderNewSMP)
% [JSRad,diagnosticJSR,sequenceJSR]=JSR_v14(A,MaxNumIter,precisionJSR,UseNorm, ...
% max(floor(log(s(1))/log(s(3))),2),[1,2,Inf],2, ...
% max(floor(log(s(1))/log(s(3))),2),3,[0,1],precisionJSR,0,0,10^-9,1,1000,1,0);
%
%  Output:
%
%    JSRad          Joint Spectral Radius lower and upper bound estimate
%
%    diagnosticJSR  Struct containg the following information:
%          text           How the algorithm ended
%          time           Total time for computations
%          norma          Norm selected by the algorithm among values
%                          defined in 'norme' and used from step 1 untill
%                          step 'StartingStep' to evaluate the
%                          upperbound beta.
%          FinalStep      Number of the last step studied by the algorithm
%          code           Code associated with 'text':
%                          -1 -> error
%                           0 -> max number of steps reached without
%                                achieving the required precision
%                           1 -> required precision achieved
%                           2 -> all branches are dead
%                           3 -> SMP founded, special case
%          branchesAlive  Vector containing in postion i the number of
%                          active branches at step i.
%          rhoF_i         Vector containing in position 'i' value rho_i(F)
%                          i.e. lower bound of the value of the Joint
%                          Spectral Radius at step i ( left side of the
%                          double inequality).
%         sequenceRhoF_i  Cell containing in postion i the sequence of
%                          matrix indices associated to the product of
%                          length i with maximal spectral radius among all
%                          the products of length i.
%         alfa            Vector containing in postion i the value of the
%                          lower bound for the Joint Spectral Radius
%                          evaluated at step i.
%         beta            Vector containing in postion i the value of the
%                          upper bound for the Joint Spectral Radius
%                          evaluated at step i through the norm specified in
%                          UseNorm.
%         posAlfa        Vector containing in postion i reference to the
%                         position of the matrix with max spectral radius
%                         at step i. In particular at step i if posAlfa(i)==0
%                         it means that we are still considering the previous
%                         estimate, otherwise  is
%                         P.((sprintf('L%d', i))).
%                         ((['A' int2str(posAlfa(i))]))
%                         the product of length i with maximal spectral
%                         radius
%         posBeta        Vector containing in postion i reference to the
%                         position of the matrix with maximal norm at step i.
%                         In particular at step i if posBeta(i)==0 it means
%                         we are still considering the previous estimate,
%                         otherwise: P.(['L' int2str(M.(sprintf('set%d', i))
%                         (posBeta(i)))]).(['A' int2str(branch.
%                         (sprintf('es%d',i))(M.(sprintf('set%d', i))
%                         (posBeta(i)),posBeta(i)))]) is the product with
%                         maximal norm;
%                         M.(sprintf('set%d', i))(posBeta(i)) is its length;
%                         posBeta(i) is the column in branch.(['es'
%                         int2str((i))])
%                         that contain, in row M.(sprintf('set%d', i))(posBeta(i)),
%                         the position inside P.(['L' int2str(M.(sprintf('set%d', i))
%                         (posBeta(i)))]) of this product.
%         normF_i        Vector containing in position 'i' value norm_i(F)
%                          i.e. upper bound of the value of the Joint
%                          Spectral Radius at step i ( right side of the
%                          double inequality).
%        sequenceNormF_i Cell containing in position i the sequence of
%                          matrix indices associated to the product of
%                          length i with minimal norm among all
%                          the products of length i.
%         z              Matrix containing as columns the vectors that
%                         define the unit ball of a Rectangular Norm. For
%                         instance z(:,LastNormStep(i)) is the vector defining the
%                         Rectangular Norm at step i.
%         Z              Matrices that define the Ellipsoidal Norm. For
%                         instance Z(:,:,LastNormStep(i)) is the matrix
%                         that define the Ellipsoidal Norm at step i.
%         SDPworked      Vector containing in position i value 1 if
%                         Semidefinite Programming worked at step i.
%                         0 otherwise.
%   sequenceJSR          Sequence of matrix indices associated to the
%                         candidate SMP.
%  Global variables
%
%  To repick the following global variables it suffices to write in the
%  command line at the end of the computations:
%  'global name_of_the_variable_to_be_repicked'
%  To repick all global variables write:
%  global P branch dry cutted word alfa beta N M Pcut z Z ...
%      posAlfa posBeta right left SDPworked oldAlfa LastNormStep;
%
%         P              Structure containing all products of the semigroup.
%                         For instance P.L3.A2 is the second matrix of
%                         length 3 belonging to the semigroup
%         branch         Matrices containing as columns the structure of
%                         every product. For instance branch.es3(:,1)=[2;4;5]
%                         it means that product P.L3.A5 has P.L2.A4 and
%                         P.L1.A2 as parents of length 2 and 1 respectively.
%         dry            Matrices that contain dry branches. For instance
%                         dry.branch3 is a 3*n matrix. n is equal to the
%                         number of products of length 3 with norm < alfa(i)
%         cutted         Matrices that contain cutted words. For instance
%                         cutted.word3 is a 3*n matrix. n is equal to the
%                         number of products of length 3 with norm < alfa(i)
%         word           Matrices containing as columns the "binary tree"
%                         i.e. the form of the product we have constructed
%         alfa           Vector containing in postion i the value of the
%                         lower bound for the Joint Spectral Radius
%                         evaluated at step i. These values as to be
%                         multiplied for oldAlfa to get the right values
%         beta           Vector containing in postion i the value of the
%                         upper bound for the Joint Spectral Radius
%                         evaluated at step i through the norm specified in
%                         UseNorm. These values as to be
%                         multiplied for oldAlfa to get the right values
%         N              Structure containing values of the norm of any
%                         product. For instance N.(['step'
%                         int2str(LastNormStep(4))]).L3.A5
%                         contains value of the norm of P.L3.A5 evaluated
%                         at step 4.
%         M              Structure in which we will store for any product
%                         of length 'i' the specific subproduct with
%                         minimal value of the norm >= alfa(i),
%                         in particular we will store just  its length
%                         (obs. the value of the norm can be repicked
%                         inside struct N.(sprintf('step%d',LastNormStep(i)).
%                         (['L' int2str(value inside the column of
%                         M.(sprintf('set%d', i)) we are studying)]).
%                         (['A' int2str(branch.(sprintf('es%d', i))
%                         (value inside the column of M.set'i' we
%                         are studying,same column of M.set'i'))]),
%                         to know the specific product of matrices we are
%                         considering we have to use in similar way
%                         information stored in struct 'branch' and 'word').
%         Pcut           Structure containing all products of the semigroup
%                         with norm strictly less then a certain alfa(i)
%         z              Matrix containing as columns the vectors that
%                         define the unit ball of a Rectangular Norm. For
%                         instance z(:,LastNormStep(i)) is the vector defining the
%                         Rectangular Norm at step i.
%         Z              Matrices that define the Ellipsoidal Norm. For
%                         instance Z(:,:,LastNormStep(i)) is the matrix
%                         that define the Ellipsoidal Norm at step i.
%         posAlfa        Vector containing in postion i reference to the
%                         position of the matrix with max spectral radius
%                         at step i. In particular at step i if posAlfa(i)==0
%                         it means that we are still considering the previous
%                         estimate, otherwise  is
%                         P.((sprintf('L%d', i))).
%                         ((['A' int2str(posAlfa(i))]))
%                         the product of length i with maximal spectral
%                         radius
%         posBeta        Vector containing in postion i reference to the
%                         position of the matrix with maximal norm at step i.
%                         In particular at step i if posBeta(i)==0 it means
%                         we are still considering the previous estimate,
%                         otherwise: P.(['L' int2str(M.(sprintf('set%d', i))
%                         (posBeta(i)))]).(['A' int2str(branch.
%                         (sprintf('es%d',i))(M.(sprintf('set%d', i))
%                         (posBeta(i)),posBeta(i)))]) is the product with
%                         maximal norm;
%                         M.(sprintf('set%d', i))(posBeta(i)) is its length;
%                         posBeta(i) is the column in branch.(['es'
%                         int2str((i))])
%                         that contain, in row M.(sprintf('set%d', i))(posBeta(i)),
%                         the position inside P.(['L' int2str(M.(sprintf('set%d', i))
%                         (posBeta(i)))]) of this product.
%         right          Last upper bound for the Joint Spectral Radius
%                         obtained at the end of a bisection technique
%                         during Semidefinite Programming.
%                         right(i) is this estimate at step i.
%         left           Last lower bound for the Joint Spectral Radius
%                         obtained at the end of a bisection technique
%                         during Semidefinite Programming.
%                         left(i) is this estimate at step i.
%         SDPworked      Vector containing in position i value 1 if
%                         Semidefinite Programming worked at step i.
%                         0 otherwise.
%         oldAlfa        Estimate of the Value of the Joint Spectral Radius
%                         with which we scale the family to avoid overflow
%                         during computations.
%         LastNormStep   In position i we have the last step in which we
%                         changed the norm.
%
%   See also ELLIPSNORM_V07, SCALINGJSR, DESCALINGJSR_V08, RHO.
%
% This software is an alpha version, please report any bug, comment or
% suggestion to antonio.cicone@univaq.it
%
% References
%    A. Cicone, V. Protasov. On the approximation of the Joint Spectra Radius, preprint, 2012.
%    G. Gripenberg. Computing the joint spectral radius. Linear Algebra Appl., vol. 234, 1996, pp 43-60.
%
% Requires
%     SeDuMi ( http://sedumi.ie.lehigh.edu/ )
%     Yalmip ( http://users.isy.liu.se/johanl/yalmip/ )
%

s=size(A);

if nargin<5 ,StartingStep=max(floor(log(s(1))/log(s(3))),2); end % we start cuttings only every log(size Matrices)/log(dimension of the family) or at least every 2
if nargin<6 ,norme=[1,2,Inf]; end
if nargin<7 ,MaxIterSDP=2; end
if nargin<8 ,OptimizeEvery=StartingStep; end
if nargin<9 ,maxNumOptimization=3; end
if nargin<10 ,askQuestion=[0,1]; end
if nargin<11 ,precisionNorm=precisionJSR; end
if nargin<12 ,NumPause=1; end
if nargin<13 ,cleanWorkspace=0; end
if nargin<14 ,epsilon=10^-9; end
if nargin<15 ,PruningEvery=1; end
if nargin<16 ,if UseNorm~='G',MaxBranchesAlive=1500; else MaxBranchesAlive=9000; end , end
if nargin<17 ,SpecialCases=1; end
if nargin<18 ,ConsiderNewSMP=0; end

tic
clear global P branch dry cutted word alfa beta N M Pcut z Z ...
    posAlfa posBeta right left SDPworked oldAlfa LastNormStep;
global P branch dry cutted word alfa beta N M Pcut z Z ...
    posAlfa posBeta right left SDPworked oldAlfa LastNormStep;
format long;
JSRad=[];
time=[];
norma=[];
i=[];
sequenceJSR=[];
branchesAlive=zeros(1,MaxNumIter);
sequenceRhoF_i=cell(1,MaxNumIter);
rhoF_i=zeros(1,MaxNumIter);
sequenceNormF_i=cell(1,MaxNumIter);
normF_i=zeros(1,MaxNumIter);
diagnosticJSR.time=-1;
diagnosticJSR.norma=NaN;
diagnosticJSR.FinalStep=NaN;
diagnosticJSR.code=-1;
diagnosticJSR.branchesAlive=0;
diagnosticJSR.rhoF_i=NaN;
diagnosticJSR.sequenceRhoF_i=NaN;
diagnosticJSR.alfa=NaN;
diagnosticJSR.beta=NaN;
diagnosticJSR.posAlfa=NaN;
diagnosticJSR.posBeta=NaN;
diagnosticJSR.normF_i=NaN;
diagnosticJSR.sequenceNormF_i=NaN;
diagnosticJSR.z=NaN;
diagnosticJSR.Z=NaN;
diagnosticJSR.SDPworked=NaN;
JSRad=NaN;

if  maxNumOptimization>0
    lengthZ=min(StartingStep+OptimizeEvery*(maxNumOptimization-1),MaxNumIter);
    if  UseNorm=='R'
        z=zeros(s(1),lengthZ);
    elseif UseNorm=='E'
        Z=zeros(s(1),s(1),lengthZ);
    end
end

%          Initial checkings         %

i=1;

if StartingStep<=1
    diagnosticJSR.text='Please choose a value greater or equal 2 as StartingStep';
    disp(diagnosticJSR.text);
    return
end

if s(1)~=s(2) % checking that matrices are square ones
    diagnosticJSR.text='Attention! No Square Matrices';
    disp(diagnosticJSR.text);
    return
end

if size(s,2)==2 % single matrix case
    disp('Joint Spectral radius equal to');
    JSRad=rho(A);
    word=1;
    diagnosticJSR.text='Single Matrix Case';
    disp(diagnosticJSR.text);
    diagnosticJSR.time=toc;
    diagnosticJSR.FinalStep=1;
    diagnosticJSR.code=3;
    diagnosticJSR.branchesAlive=1;
    return
end

%          Special cases         %

if SpecialCases==1

    % All Hermitian matrices --> JSR=max(rho(A_i)) --> OK (since we are
    % considering also norm 2) ref. Lemma 2 in "Guglielmi, Manni, Vitale - 2010

    % - Convergence analysis of C2 Hermite interpolatory subdivision
    % schemes by explicit joint spectral radius formulas"

    % Cases of 2 matrices
    if s(3)==2
        % 2 matrices s.t. F={A,A'} -> JSR=rho(A*A')^(1/2)
        % Lemma 3 in "Guglielmi, Manni, Vitale - 2010
        % - Convergence analysis of C2 Hermite interpolatory subdivision
        % schemes by explicit joint spectral radius formulas"
        if isequal(A(:,:,1),A(:,:,2)')
            JSRad=rho(A(:,:,1)*A(:,:,2))^(1/2);
            diagnosticJSR.text='Case A(:,:,2)=A(:,:,1)''';
            disp(diagnosticJSR.text);
            diagnosticJSR.time=toc;
            diagnosticJSR.norma=2;
            diagnosticJSR.FinalStep=1;
            diagnosticJSR.code=3;
            diagnosticJSR.branchesAlive=1;
            return;
        end
        % Subcases of 2 matrices 2x2
        if s(1)==2
            % 2 matrices s.t. A(:,:,1)=[a b; c d], A(:,:,2)=[a -b; -c d]
            % ref. Theorem 4 in "Guglielmi, Manni, Vitale - 2010 - Convergence
            %  analysis of C2 Hermite interpolatory subdivision schemes by
            % explicit joint spectral radius formulas"
            if (A(1,1,1)==A(1,1,2) && A(2,2,1)==A(2,2,2) && ...
                    A(1,2,1)==-A(1,2,2) && A(2,1,1)==-A(2,1,2))
                if A(1,2,1)*A(2,1,1)>=0
                    JSRad=rho(A(:,:,1));
                    diagnosticJSR.text='Case A(:,:,1)=[a b; c d], A(:,:,2)=[a -b; -c d] with b*c>=0';
                    disp(diagnosticJSR.text);
                    diagnosticJSR.time=toc;
                    diagnosticJSR.FinalStep=1;
                    diagnosticJSR.code=3;
                    diagnosticJSR.branchesAlive=1;
                    return;
                else
                    JSRad=rho(A(:,:,1)*A(:,:,2))^(1/2);
                    diagnosticJSR.text='Case A(:,:,1)=[a b; c d], A(:,:,2)=[a -b; -c d] with b*c<0';
                    disp(diagnosticJSR.text);
                    diagnosticJSR.time=toc;
                    diagnosticJSR.FinalStep=2;
                    diagnosticJSR.code=3;
                    diagnosticJSR.branchesAlive=1;
                    return;
                end
            end
            % 2 matrices s.t. A(:,:,1)=[a b; c d], A(:,:,2)=[d c; b a] i.e.
            % for T=[0 1;1 0] we have A(:,:,1)=T*A(:,:,2)*T
            % ref. Theorem 5 in "Guglielmi, Manni, Vitale - 2010 - Convergence
            %  analysis of C2 Hermite interpolatory subdivision schemes by
            % explicit joint spectral radius formulas"
            T=[0 1;1 0];
            if isequal(A(:,:,1),T*A(:,:,2)*T)
                if abs(A(1,1,1)-A(2,2,1))>=abs(A(1,2,1)-A(2,1,1))
                    JSRad=rho(A(:,:,1));
                    diagnosticJSR.text='Case A(:,:,1)=[a b; c d], A(:,:,2)=[d c; b a] with |a-d|>=|b-c|';
                    disp(diagnosticJSR.text);
                    diagnosticJSR.time=toc;
                    diagnosticJSR.FinalStep=1;
                    diagnosticJSR.code=3;
                    diagnosticJSR.branchesAlive=1;
                    return;
                else
                    JSRad=rho(A(:,:,1)*A(:,:,2))^(1/2);
                    diagnosticJSR.text='Case A(:,:,1)=[a b; c d], A(:,:,2)=[d c; b a] with |a-d|<|b-c|';
                    disp(diagnosticJSR.text);
                    diagnosticJSR.time=toc;
                    diagnosticJSR.FinalStep=2;
                    diagnosticJSR.code=3;
                    diagnosticJSR.branchesAlive=1;
                    return;
                end
            end
        end
    end
end

starText='*******************************************************************************';
barraText='_________________________________________________________________________';


%      Initialization of variables and nonnegativity checking        %

nonnegative=1==1;
for ii = 1:s(3)% We pass matrices conteined in A into distinct arrays of
    % structure P this will allow us to delete matrices without messing up
    % with indexing of all the other products.
    P.L1.(sprintf('A%d',ii))=A(:,:,ii); % P is a structure
    % containing all products. In particular we memorize in 'P.L5.Axx'
    % all products of length '5'
    if UseNorm=='R'
        nonnegative_i=A(:,:,ii)>=0;
        sumColumn_i=sum(nonnegative_i);
        sumTot_i=sum(sumColumn_i);
        if sumTot_i~=s(1)^2
            nonnegative=1==0;
        end
    end
end
if UseNorm=='R' && not(nonnegative)
    diagnosticJSR.text=' ATTENTION! Rectangular Norms work only with families of nonnegative matrices';
    disp(diagnosticJSR.text);
    return;
end

Pcut.L1=[]; % Here we will store any product of length '1' with norm
% strictly less then a certain 'alfa^1'
word.s1=1:s(3);% we are memorizing in the "binary" tree
% the form of the product we have constructed
branch.es1=1:s(3);% we are memorizing the position
% of P.(sprintf('L%d', i)).(['A' int2str(z)]), the product of
% lenght one we have constructed, where 'i' coincide with
% the row of branch.(sprintf('es%d', i))(i,z) we are considering
dry.branch1=[];
cutted.word1=[];
branchesAlive(1)=s(3);


%          Evaluation of initial alfa          %


alfa=zeros(1,s(3));
for ii= 1:s(3)
    alfa(ii)=rho(P.L1.(sprintf('A%d',ii)));
end
[alfa,posAlfa]=max(alfa);
% Matrix with max spectral radius is P.L1.(sprintf('A%d',posAlfa))
rhoF_i(i)=alfa;
sequenceRhoF_i{i}=posAlfa;


%            Scaling of the family            %

%                      +

%          Evaluation of initial beta         %

LastNormStep(1)=1;
if abs(alfa)>=10^-15 % alfa is different from zero
    oldAlfa=alfa;% We store the scaling value
    beta=[];
    lenNorme=length(norme);
    maxbeta=zeros(1,lenNorme);
    posMaxBeta=zeros(1,lenNorme);
    betaNorma_j=zeros(s(3),lenNorme);
    for ii= 1:s(3)
        P.L1.(sprintf('A%d',ii))=P.L1.(sprintf('A%d',ii))/alfa;
        for j= 1:lenNorme
            % we store the value of a norm
            betaNorma_j(ii,j)=norm(P.L1.(sprintf('A%d',ii)),norme(j));
        end
    end

    for j= 1:lenNorme
        [maxbeta(j),posMaxBeta(j)]=max(betaNorma_j(:,j));
    end
    [beta,pos]=min(maxbeta);
    normF_i(i)=beta*oldAlfa;
    posBeta=posMaxBeta(pos);
    sequenceNormF_i{i}=posBeta;
    norma=norme(pos);
    % Matrix with max norm "norma" is P.L1.(['A'int2str(posBeta)])
    alfa=1;% alfa becames 1 due to scaling
    for ii= 1:s(3)
        N.step1.L1.(sprintf('A%d',ii))=betaNorma_j(ii,pos);
    end    
    precisionJSR=precisionJSR/oldAlfa;
else % alfa is almost zero
    oldAlfa=1;
    beta=[];
    lenNorme=length(norme);
    maxbeta=zeros(1,lenNorme);
    posMaxBeta=zeros(1,lenNorme);
    betaNorma_j=zeros(s(3),lenNorme);
    for ii= 1:s(3)
        for j= 1:lenNorme
            % we store the value of a norm
            betaNorma_j(ii,j)=norm(P.L1.(sprintf('A%d',ii)),norme(j));
        end
    end

    for j= 1:lenNorme
        [maxbeta(j),posMaxBeta(j)]=max(betaNorma_j(:,j));
    end
    [beta,pos]=min(maxbeta);
    normF_i(i)=beta;
    posBeta=posMaxBeta(pos);
    sequenceNormF_i{i}=posBeta;
    norma=norme(pos);
    % Matrix with max norm "norma" is P.L1.(['A'int2str(posBeta)])
    for ii= 1:s(3)
        N.step1.L1.(sprintf('A%d',ii))=betaNorma_j(ii,pos);
    end    
end


%          Changing Norm for initial evaluations         %

if askQuestion(1)==1
    normChangedUsingNorma=0;
    while normChangedUsingNorma==0
        fprintf('The algorithm is going to use Norm %1.0f for initial evaluations\n',...
            norma);
        if isempty(find(norme==2))
            disp('Remember that norm 2 is the best choice to study Hermitian sets.')
        end
        disp('Change Norm?')
        changeNorm=input('No -0- or Yes -1-\n<< ');
        if changeNorm==1
            normaSuggested=input('Please insert the desired norm choosing among: 1, 2 and Inf\n<< ');
            for ii=1:length(norme)
                if norme(ii)==normaSuggested
                    beta=maxbeta(ii);
                    normF_i(i)=beta*oldAlfa;
                    posBeta=posMaxBeta(ii);
                    sequenceNormF_i{i}=posBeta;
                    normChangedUsingNorma=1;
                    break
                end
            end
            if normChangedUsingNorma==0
                fprintf('\nThe Norm %1.0f you entered is not a good one.\nPlease, try again\n',...
                    normaSuggested);
            else
                norma=normaSuggested;
            end
        else
            normChangedUsingNorma=1;
        end
    end
end


if askQuestion(1)==1
    if norma==Inf && UseNorm=='E' && nonnegative
        disp('Probably it should be better to use rectangular norm')
        disp('Do you want to go on anyway?')
        cont=input('Yes -1- or No -0-\n<< ');
        if cont==0
            diagnosticJSR.text='Algorithm interrupted by the user';
            disp(diagnosticJSR.text);
            return
        end
    end

    if norma==1
        if UseNorm=='E' && nonnegative
            disp('Probably it should be  better to transpose the family and use rectangular norm')
            disp('Do you want to go on anyway?')
            cont=input('Yes -1- or No -0-\n<< ');
            if cont==0
                diagnosticJSR.text='Algorithm interrupted by the user';
                disp(diagnosticJSR.text);
                return
            end
        end
        if UseNorm=='R'
            disp('Probably it should be  better to transpose the family')
            disp('Do you want to go on anyway?')
            cont=input('Yes -1- or No -0-\n<< ');
            if cont==0
                diagnosticJSR.text='Algorithm interrupted by the user';
                disp(diagnosticJSR.text);
                return
            end
        end
    end

    if norma==2 && UseNorm=='R'
        disp('Probably it should be  better to use an ellipsoidal norm')
        disp('Do you want to go on anyway?')
        cont=input('Yes -1- or No -0-\n<< ');
        if cont==0
            diagnosticJSR.text='Algorithm interrupted by the user';
            disp(diagnosticJSR.text);
            return
        end
    end
end


if abs(alfa-beta)<=precisionJSR % for the double inequality we have
    % identified the Spectrum Maximizing Product (SMP)
    % up to prescribed precision
    if oldAlfa~=1
        precisionJSR=precisionJSR*oldAlfa;
        DescalingJSR_v08(i)
    end
    diagnosticJSR.text=['Precision required attained after 1 step using norm ' num2str(norma)];
    disp(diagnosticJSR.text);
    disp('Joint Spectral radius among');
    JSRad=[alfa,beta]
    disp('Lower and upper bound given by');
    fprintf('rho(A(:,:,%1.0d)) and norm(A(:,:,%1.0d),%1.0d)\n',...
        word.s1(posAlfa),word.s1(posBeta),norma);
    diagnosticJSR.time=toc;
    diagnosticJSR.norma=norma;
    diagnosticJSR.FinalStep=1;
    diagnosticJSR.code=1;
    diagnosticJSR.branchesAlive=s(3);
    return
end



%                               Main loop                                 %

FinalText='\n    MaxNumIter reached';
if UseNorm=='G'
    NormUsedText=sprintf(' using Gripenberg algorithm with norm %d', norma);
elseif UseNorm=='R'
    NormUsedText=sprintf(' using Rectangular norm and norm %d', norma);
elseif UseNorm=='E'
    NormUsedText=sprintf(' using Ellipsoidal norm and norm %d', norma);
else
    disp('Value')
    disp(UseNorm)
    disp('of UseNorm not recognized')
    disp('Please choose a correct value for UseNorm among:')
    disp('  ''E'' for Ellipsoidal Norms')
    disp('  ''R'' for Rectangular Norms')
    disp('  ''G'' for Gripenberg')
    diagnosticJSR.text='Value of UseNorm not recognized';
    return
end

if askQuestion(1)==1
    fprintf(['\nThe method is' NormUsedText '\n']);
    pause(2)
end

if MaxNumIter==1
    if oldAlfa~=1
        precisionJSR=precisionJSR*oldAlfa;
        DescalingJSR_v08(i)
    end
    disp('Please augment value of MaxNumIter');
    fprintf('Joint Spectral Radius greater or equal to %d and less or equal to %d',alfa,beta);
    JSRad=[alfa,beta];
    disp('Lower and upper bound given by matrices');
    P.L1.(['A' sprintf('%d',posAlfa)])
    P.L1.(['A' sprintf('%d',posBeta)])
    fprintf('which are respectively the %1.0f and %1.0f matrix of the family\n',...
        word.s1(posAlfa),word.s1(posBeta));
    diagnosticJSR.text='Value of MaxNumIter equal to 1';
    diagnosticJSR.time=toc;
    diagnosticJSR.norma=norma;
    diagnosticJSR.FinalStep=1;
    diagnosticJSR.code=0;
    diagnosticJSR.branchesAlive=s(3);
    return;
end

time(i)=toc;

if askQuestion(2)==1
    if alfa(i)>=10^-12
        relativeError=(beta(i)-alfa(i))/alfa(i);
    else
        relativeError=(beta(i)-alfa(i))/beta(i);
    end
    fprintf('\n    <<<  Step %1.0f of %1.0f  >>>\n',i,MaxNumIter);
    fprintf('\n Joint Spectral radius untill now is among {%4.15f, %4.15f}\n Relative error is around %4.15f \n',...
        alfa(i)*oldAlfa,beta(i)*oldAlfa,relativeError);
    fprintf(['\n' starText '\n\n   %1.0f of %1.0f active branches at step %3.0f\n\n' starText '\n'],...
        s(3)^i,s(3)^i,i);
end

for i = 2:MaxNumIter

    if NumPause~=0
        % we ask any NumPause step if we want to go on or not
        if mod(i,NumPause)==0
            if askQuestion(1)==1
                fprintf('\n After %1.0f of %1.0f steps\n Joint Spectral radius among {%1.15f, %1.15f}\n',...
                    i-1,MaxNumIter,alfa(end)*...
                    oldAlfa,beta(end)*oldAlfa);
                fprintf('number of branches alive %1.0d \n\n',branchesAlive(i-1));
                disp(' Do you want to go on?')
                cont=input(' Yes -1- or No -0-\n<< ');
                if cont==0
                    i=i-1;
                    diagnosticJSR.code=0;
                    break;
                end
            else
                if askQuestion(2)==1
                    fprintf('\n After %1.0f of %1.0f steps\n Joint Spectral radius among {%1.15f, %1.15f}\n',...
                        i-1,MaxNumIter,alfa(length(alfa))*...
                        oldAlfa,beta(length(beta))*oldAlfa);
                    fprintf('number of branches alive %1.0d \n\n',branchesAlive(i-1));
                end
                if i<=StartingStep
                    cont=1;
                elseif i>StartingStep && branchesAlive(i-1)<=MaxBranchesAlive
                    cont=1;
                else
                    cont=0;

                end
                if cont==0
                    i=i-1;
                    diagnosticJSR.code=0;
                    FinalText='\n    Number of MaxBranchesAlive reached';
                    break;
                end
            end
        end
    end

    if askQuestion(2)==1
        fprintf('\n    <<<  Step %1.0f of %1.0f  >>>\n',i,MaxNumIter);
    end


    %           Initialization of variables at step i            %

    word.(sprintf('s%d', i))=[]; % it generates a new variable
    % for any new 'i'. To repick this variable it suffices to write
    % "word.(sprintf('s%d', i))"
    branch.(sprintf('es%d', i))=[];
    % We prepare to memorize dry branches of length 'i'
    dry.(sprintf('branch%d', i))=[];
    cutted.(sprintf('word%d', i))=[];
    k=1;
    Pcut.((sprintf('L%d', i)))=[]; % Here we will store any
    % product of length 'i' with norm strictly less then a certain 'alfa^i'
    sBranch_old=size(branch.(sprintf('es%d', i-1)));% number of products of length 'i-1'
    % we are considering i.e. all the elements of L_{i-1}
    rho_new=zeros(1,sBranch_old(2)*s(3));
    while k<=sBranch_old(2)% we have to consider only the matrices from
        % P.(sprintf('L%d',i-1)).(['A' int2str(z)])
        % generated in the previous passage 'i-1' (they are all and
        % only the ones that belong to set L_{i-1})
        word.(sprintf('s%d', i))(1:i,(1:s(3))+(k-1)*s(3))=...
            [word.(sprintf('s%d',i-1))(:,k)*ones(1,s(3));1:s(3)];% we are memorizing
        % in the "binary" tree the form of the product we have constructed
        branch.(sprintf('es%d', i))(1:i,(1:s(3))+(k-1)*s(3))= [branch. ...
            (sprintf('es%d', i-1))(:,k)*ones(1,s(3));(1:s(3))+(k-1)*s(3)];
        % we are memorizing the position of P.(sprintf('L%d', i)).(['A'
        % int2str(z)]), the product we have constructed, where 'i'
        % coincide with the last row of branch.(sprintf('es%d', i))(i,z)
        % we are considering
        Pktemp=P.(sprintf('L%d',i-1)).(sprintf('A%d',branch. ...
            (sprintf('es%d', i-1))(i-1,k)));
        for j = 1:s(3)% we consider a previous product of length 'i-1'
            % and we multiply it for all matrices in the family
            % F={A_j}_{j\in {1,...,s(3)}}
            Ptemp=Pktemp*P.L1.(sprintf('A%d',j));
            P.((sprintf('L%d', i))).(sprintf('A%d',(k-1)*s(3)+j))=Ptemp;
            % we memorize the specific product of length 'i'
            rho_new((k-1)*s(3)+j)=rho(Ptemp)^(1/i);
        end
        k=k+1;
    end


    %                 Estimate of alfa at step i                 %


    [rhoF_i(i),posRhoF_i]=max(rho_new);
    [alfa(i),positionMaxSpectralRadius]=max([alfa(i-1),rhoF_i(i)]);
    rhoF_i(i)=rhoF_i(i)*oldAlfa;% descaling
    sequenceRhoF_i{i}=word.(sprintf('s%d', i))(:,posRhoF_i);
    if positionMaxSpectralRadius==2
        posAlfa(i)=posRhoF_i;
    else
        posAlfa(i)=0;
    end

    %*********************************************************************
    % NB If posAlfa(i)==0 it means that we are still considering the previous
    % estimate, otherwise  is P.((sprintf('L%d', i))).((['A' int2str(posAlfa(i))])) the product of
    % length i with maximal spectral radius
    %*********************************************************************


    %         %                   Scaling of the family                    %
    %
    %
    %     if posAlfa(i)~=0 %we have a new candidate Spectrum Maximizing Product
    %
    %     end


    %                   i>=StartingStep Section                  %


    % We introduce a bisection that allows us to identify a good erre
    % approximating the JSR:

    M.((sprintf('set%d', i)))=[];% Here we will store for
    % any product of length 'i' the specific subproduct with minimal
    % value of the norm >= alfa(i), in particular we will store just
    % its length (obs. the value of the norm can be repicked inside struct
    % N.(sprintf('step%d',LastNormStep(i)).(['L' int2str(value inside
    % the column of M.set'i' we are studying)]).(['A' int2str(branch.
    % (sprintf('es%d', i))(value inside the column of M.set'i' we
    % are studying, same column of M.set'i'))]), for the specific
    % product of matrices we will use in similar way information
    % stored in struct 'branch' and 'word').

    if i>=StartingStep
        LastNormStep(i)=LastNormStep(i-1);
        if (mod(i-StartingStep,OptimizeEvery)==0 && maxNumOptimization>0 ...
                && UseNorm~='G') || (ConsiderNewSMP==1 && posAlfa(i)~=0 ...
                && maxNumOptimization>0 && UseNorm~='G')
            maxNumOptimization=maxNumOptimization-1;
            check=0==0;
            kk=1; %OBS: change to kk=0 if you want to check if the upperbound is feasible
            left(i)=alfa(i);
            right(i)=beta(i-1);
            if abs(alfa(i)-beta(i-1))<=precisionJSR
                FinalText='\n    Precision required attained';
                diagnosticJSR.code=1;                
                break;
            end



            %            Semidefinite Optimization at step i             %


            SDPworked(i)=0;
            numUpIncrements=0;
            while check && kk<=MaxIterSDP
                % We stop either if we have got a good precision or
                % if we achieved the maximal number of iterations


                %             Using Rectangular Norm             %


                if  UseNorm=='R'
                    if kk==0 % we start checking the upperbound to be feasible
                        erre=right(i);
                    else
                        erre=(left(i)+right(i))/2;
                    end
                    [diagnostic,possibleZ]=RectNorm_v07(i,erre,epsilon,...
                        StartingStep,precisionJSR);
                    if diagnostic.problem==(1==0)%i.e. the value erre is a good one
                        right(i)=erre;
                        if askQuestion(2)==1
                            fprintf('+ Right bound fixed to %4.15f\n\n',...
                                right(i)*oldAlfa);
                        end
                        SDPworked(i)=1;
                        z(:,i)=possibleZ;
                    else
                        if kk==0  % if for kk=0 we do not find a rectangular norm we exit
                            fprintf('Initial upper bound %4.15f too much tight\n',...
                                erre*oldAlfa);
                            if askQuestion(1)==1
                                disp('Do you prefer to:')
                                disp('0 --> exit')
                                disp('1 --> try incrementing initial upper bound')
                                upperTight=input('<< ');
                                if upperTight==1
                                    right(i)=right(i)+100.5/oldAlfa;
                                    kk=kk-1;
                                else
                                    diagnosticJSR.text='Initial upper bound too much tight, algorithm interrupted by the user';
                                    return
                                end
                            else
                                if numUpIncrements<1
                                    numUpIncrements=numUpIncrements+1;
                                    fprintf('\nWe try incrementing the upper bound for the %1.0d time\n\n\n',...
                                        numUpIncrements);
                                    right(i)=right(i)+10/oldAlfa;
                                    kk=kk-1;
                                else
                                    fprintf('\nInitial upper bound %4.15f still too much tight\n',...
                                        erre*oldAlfa);
                                    diagnosticJSR.text='Initial upper bound too much tight';
                                    return
                                end
                            end
                        else
                            left(i)=erre;
                            if askQuestion(2)==1
                                fprintf('- Left bound fixed to %4.15f\n\n',...
                                    left(i)*oldAlfa);
                            end
                        end
                    end
                    if abs(alfa(i)-right(i))<=precisionJSR
                        if SDPworked(i)==1
                            break;
                        else
                            warning('Precision required on JSR attained, but Rectangular norm not yet found');
                        end
                    end
                    if abs((left(i)-right(i))*oldAlfa)<=precisionNorm
                        if SDPworked(i)==1
                            % Precision attained and it has been obtained
                            % at least one estimate for z
                            check=1==0;
                        else % We didn't obtain any estimate for z
                            disp('No rectangular Norm')
                            disp('Please try reducing value of precisionNorm');
                            diagnosticJSR.text='No rectangular Norm founded, please try reducing value of precisionNorm';
                            return
                        end
                    end
                    kk=kk+1;
                end



                %             Using Ellipsoidal Norm             %


                if  UseNorm=='E'
                    % we start checking the upperbound to be feasible
                    if kk==0
                        erre=right(i);
                    else
                        erre=(left(i)+right(i))/2;
                    end
                    [diagnostic,possibleZ]=EllipsNorm_v07(i,erre,epsilon,...
                        StartingStep,precisionJSR);
                    if diagnostic.problem==(1==0)%i.e. the value erre is a good one
                        right(i)=erre;
                        if askQuestion(2)==1
                            fprintf('+ Right bound fixed to %4.15f\n\n',...
                                right(i)*oldAlfa);
                        end
                        SDPworked(i)=1;
                        Z(:,:,i)=possibleZ;
                    else
                        % if for kk=0 we do not find an Ellipsoidal norm we exit
                        if kk==0
                            fprintf('Initial upper bound %4.15f too much tight\n',...
                                erre*oldAlfa);
                            if askQuestion(1)==1
                                disp('Do you prefer to:')
                                disp('0 --> exit')
                                disp('1 --> try incrementing initial upper bound')
                                upperTight=input('<< ');
                                if upperTight==1
                                    right(i)=right(i)+0.5/oldAlfa;
                                    kk=kk-1;
                                else
                                    diagnosticJSR.text='Initial upper bound too much tight, algorithm interrupted by the user';
                                    return
                                end
                            else
                                if numUpIncrements<1
                                    numUpIncrements=numUpIncrements+1;
                                    fprintf('\nWe try incrementing the upper bound for the %1.0d time\n\n\n',...
                                        numUpIncrements);
                                    right(i)=right(i)+10/oldAlfa;
                                    kk=kk-1;
                                else
                                    fprintf('\nInitial upper bound %4.15f still too much tight\n',...
                                        erre*oldAlfa);
                                    diagnosticJSR.text='Initial upper bound too much tight';
                                    return
                                end
                            end
                        else
                            left(i)=erre;
                            if askQuestion(2)==1
                                fprintf('- Left bound fixed to %4.15f\n\n',...
                                    left(i)*oldAlfa);
                            end
                        end
                    end
                    if abs(alfa(i)-right(i))<=precisionJSR
                        if SDPworked(i)==1
                            break;
                        else
                            warning('Precision required on JSR attained, but Ellipsoidal norm not yet found');
                        end
                    end
                    if abs((left(i)-right(i))*oldAlfa)<=precisionNorm
                        if SDPworked(i)==1
                            % Precision attained and it has been obtained
                            % at least one estimate for Z
                            check=1==0;
                        else % We didn't obtain any estimate for Z                            
                            if askQuestion(1)==1
                                disp('No Ellipsoidal Norm founded');
                                disp('It might be better to restart reducing value of precisionNorm');                                
                                disp('Do you prefer to:');
                                disp('0 --> restart');
                                disp('1 --> go on anyway');
                                reStart=input('<< ');
                                if reStart==1
                                    break
                                else
                                    diagnosticJSR.text='No Ellipsoidal Norm founded, please try reducing value of precisionNorm';
                                    return
                                end
                            else
                                if norm(SDPworked,1)>0
                                    break
                                else
                                    diagnosticJSR.text='No Ellipsoidal Norm founded, please try reducing value of precisionNorm';
                                    return
                                end
                            end
                        end
                    end
                    kk=kk+1;
                end
            end
            if  SDPworked(i)==1
                LastNormStep(i)=i;
            end
        end


        %               Evaluation of Norm at step i                 %



        %              Using Gripenberg idea             %

        if  UseNorm=='G'
            for j = i
                field_j=fieldnames(P.(sprintf('L%d',j)));
                numProd_j=size(field_j);
                for jj = 1:numProd_j(1)
                    NameMatrix_jj=char(field_j(jj));
                    N.step1.(sprintf('L%d',j)). ...
                        (NameMatrix_jj)=norm(P.(sprintf('L%d',j)). ...
                        (NameMatrix_jj),norma)^(1/j);
                end
            end
        end


        %             Using Rectangular Norm             %

        if  UseNorm=='R'
            if LastNormStep(i)==i % We have a new estimate for z
                % Now we store the norm, generated by vector z evaluated
                % at step 'i', of all products of all lengths
                for j = 1:i
                    field_j=fieldnames(P.(sprintf('L%d',j)));
                    numProd_j=size(field_j);
                    for jj = 1:numProd_j(1)
                        NameMatrix_jj=char(field_j(jj));
                        N.(sprintf('step%d', i)). ...
                            (sprintf('L%d',j)). ...
                            (NameMatrix_jj)=norm((P.(sprintf('L%d',j)). ...
                            (NameMatrix_jj)*z(:,i))./z(:,i),Inf)^(1/j);
                    end
                end
            else % We didn't have a new estimate for z at step i
                for j = i
                    field_j=fieldnames(P.(sprintf('L%d',j)));
                    numProd_j=size(field_j);
                    for jj = 1:numProd_j(1)
                        NameMatrix_jj=char(field_j(jj));
                        N.(sprintf('step%d',LastNormStep(i))). ...
                            (sprintf('L%d',j)). ...
                            (NameMatrix_jj)=norm((P.(sprintf('L%d',j)). ...
                            (NameMatrix_jj)*z(:,LastNormStep(i)))./z(:, ...
                            LastNormStep(i)),Inf)^(1/j);
                    end
                end
            end
        end


        %             Using Ellipsoidal Norm             %

        if  UseNorm=='E'
            if LastNormStep(i)==i % We have a new estimate for Z
                % Now we store the norm, generated by Positive definite
                % matrix Z, of all products of all lengths
                C(:,:,i)=chol(Z(:,:,i));
                invC(:,:,i)=inv(C(:,:,i));
                for j = 1:i
                    field_j=fieldnames(P.(sprintf('L%d',j)));
                    numProd_j=size(field_j);
                    for jj = 1:numProd_j(1)
                        NameMatrix_jj=char(field_j(jj));
                        N.(sprintf('step%d', i)). ...
                            (sprintf('L%d',j)). ...
                            (NameMatrix_jj)=norm(C(:,:,i)*(P. ...
                            (sprintf('L%d',j)).(NameMatrix_jj))* ...
                            invC(:,:,i),2)^(1/j);
                    end
                end
            else % We didn't have a new estimate for Z at step i
                for j = i
                    field_j=fieldnames(P.(sprintf('L%d',j)));
                    numProd_j=size(field_j);
                    for jj = 1:numProd_j(1)
                        NameMatrix_jj=char(field_j(jj));
                        N.(sprintf('step%d',LastNormStep(i))). ...
                            (sprintf('L%d',j)). ...
                            (NameMatrix_jj)=norm(C(:,:,LastNormStep(i))* ...
                            (P.(sprintf('L%d',j)).(NameMatrix_jj))* ...
                            invC(:,:,LastNormStep(i)),2)^(1/j);
                    end
                end
            end
        end


        %            Evaluation of set M and A at step i             %
        %                      with prunings                         %


        if PruningEvery~=0 && mod(i-StartingStep,PruningEvery)==0
            % we want to prune dry branches every PruningEvery

            % Now we find for every product of length 'i' the subproduct
            % with minimal norm and we create sets M, A and clean set L
            numProd_i=size(branch.(sprintf('es%d',i)));
            branchToBeCut=[];
            positionOfProductToBeCut=[];
            minimalNorms_i=[];

            if LastNormStep(i)==i || (UseNorm=='G' && i==StartingStep)
                % We have a new estimated norm
                for j = 1:numProd_i(2)
                    Mprod_j=[];
                    for jj = 1:i
                        Mprod_j=[Mprod_j,N.(sprintf('step%d',LastNormStep(i))). ...
                            (sprintf('L%d',jj)).(sprintf('A%d',branch.(sprintf('es%d',i))(jj,j)))];
                    end
                    [value,position]=min(Mprod_j);% acting in this way we
                    % obtain the first minimal value, that it is good in case
                    % of two exactly equal minimal values for two different
                    % value of 'jj' i.e. length
                    if value>alfa(i)+precisionJSR;% If this is true the product
                        % remain inside set P
                        M.(sprintf('set%d', i))=[M.(sprintf('set%d', i)),position];
                        minimalNorms_i=[minimalNorms_i,value];
                    else% Otherwise we pass it from set P (a.k.a. L) to set
                        % Pcut (a.k.a. A), we modify branch, word and dry.
                        branchToBeCut=[j,branchToBeCut];
                        positionOfProductToBeCut=[position,positionOfProductToBeCut];
                    end
                end
            else % We do not have a new estimated norm

                for j = 1:sBranch_old(2)
                    Nj=N.(sprintf('step%d',LastNormStep(i))). ...
                        (sprintf('L%d',M.(sprintf('set%d', i-1))(1,j))). ...
                        (sprintf('A%d',branch.(sprintf('es%d', i-1))(M.(sprintf('set%d', i-1))(1,j),j)));
                    for  k=1:s(3)
                        [value,position]=min([Nj, ...
                            N.(sprintf('step%d',LastNormStep(i))). ...
                            (sprintf('L%d', i)).(sprintf('A%d',branch.(sprintf('es%d',i))(i,(j-1)*s(3)+k)))]);
                        % acting in this way we obtain the first minimal value,
                        % that it is good in case of two minimal values
                        % exactly equal for two different lengths
                        if value>alfa(i)+precisionJSR % If this is true the product
                            % remain inside set P
                            if position==1 % i.e. the old estimate is still the minimal one
                                M.(sprintf('set%d', i))=[M.(sprintf('set%d', i)), ...
                                    M.(sprintf('set%d', i-1))(1,j)];
                            else
                                M.(sprintf('set%d', i))=[M.(sprintf('set%d', i)),i];
                            end
                            minimalNorms_i=[minimalNorms_i,value];
                        else % Otherwise we pass it from set P (a.k.a. L) to set
                            % Pcut (a.k.a. A), we modify branch, word and dry.
                            branchToBeCut=[(j-1)*s(3)+k,branchToBeCut];
                            if position==1 % i.e. the old estimate is still the minimal one
                                positionOfProductToBeCut=[M.(sprintf('set%d', i-1))(1,j),positionOfProductToBeCut];
                            else
                                positionOfProductToBeCut=[i,positionOfProductToBeCut];
                            end
                        end
                    end
                end
            end

            %               Dry branches pruning at step i               %

            % We want to cut dry branches, we have to start from the right
            % side otherwise we will lose the exact position among columns
            % of branch and word
            NumCuttings=length(branchToBeCut);
            for jj = 1:NumCuttings
                position=positionOfProductToBeCut(jj);% it coincides with
                % the length of the product we have to exclude
                j=branchToBeCut(jj);
                Pcut.(sprintf('L%d',position)). ...
                    (sprintf('A%d',branch.(sprintf('es%d',i))(position,j)))=P. ...
                    (sprintf('L%d',position)).(sprintf('A%d',branch.(sprintf('es%d',i))(position,j)));
                % Name of the matrix product we have to cut==
                % (['A' int2str(branch.(sprintf('es%d',i))(position,j))]);
                % We should remove this matrix from P using the command
                % P.(sprintf('L%d',position))=rmfield(P.(sprintf('L%d',position)).
                % (sprintf('A%d',branch.(sprintf('es%d',i))(position,j)));

                numDryBranch=size(dry.(sprintf('branch%d',position)));
                checkDry=1==1;
                hh=1;
                if numDryBranch~=0
                    checkDry=min(sum(abs(dry.(sprintf('branch%d',position))-...
                        branch.(sprintf('es%d',i))(1:position,j)*ones(1,numDryBranch(2)))))~=0;
                end
                if checkDry
                    dry.(sprintf('branch%d',position))=[dry.(sprintf('branch%d' ...
                        ,position)),branch.(sprintf('es%d', i))(1:position,j)];
                    cutted.(sprintf('word%d', position))=[cutted.(sprintf('word%d', ...
                        position)),word.(sprintf('s%d', i))(1:position,j)];
                end
                branch.(sprintf('es%d',i))(:,j)=[];
                word.(sprintf('s%d', i))(:,j)=[];
                % numProd_i(2)=numProd_i(2)-1;% we have deleted a column
            end


            %                 Checking of Pruning section                %


            numProd_i=size(branch.(sprintf('es%d',i)));
            numNormEvaluated=size(minimalNorms_i);
            if numProd_i(2)~=numNormEvaluated(2)
                warning('ATTENTION! something wrong in section ''Dry branches pruning''')
                diagnosticJSR.text='Something wrong in section ''Dry branches pruning''';
                return
            end


            activeBranches=size(branch.(sprintf('es%d',i)));
            branchesAlive(i)=activeBranches(2);

            if activeBranches(2)==0
                if askQuestion(2)==1
                    fprintf(['\n\n' starText '\n\n   At the end of step %1.0f all branches are dead \n\n' starText '\n'],...
                        i);
                end
                FinalText=sprintf('\n    After %d steps all branches are dead',i);
                diagnosticJSR.code=2;
                beta(i)=alfa(i)+precisionJSR;
                normF_i(i)=beta(i)*oldAlfa;
                break
            end


            %            Evaluation of set M and A at step i             %
            %                     without prunings                       %


        else % we do not prune dry branches
            numProd_i=size(branch.(sprintf('es%d',i)));
            minimalNorms_i=[];

            if LastNormStep(i)==i % We have a new estimated norm
                for j = 1:numProd_i(2)
                    Mprod_j=[];
                    for jj = 1:i
                        Mprod_j=[Mprod_j,N.(sprintf('step%d',LastNormStep(i))). ...
                            (sprintf('L%d',jj)).(sprintf('A%d',branch.(sprintf('es%d',i))(jj,j)))];
                    end
                    [value,position]=min(Mprod_j);% acting in this way we
                    % obtain the first minimal value, that it is good in case
                    % of two exactly equal minimal values for two different
                    % value of 'jj' i.e. length
                    M.(sprintf('set%d', i))=[M.(sprintf('set%d', i)),position];
                    minimalNorms_i=[minimalNorms_i,value];
                end
            else % We do not have a new estimated norm

                for j = 1:sBranch_old(2)
                    Nj=N.(sprintf('step%d',LastNormStep(i))). ...
                        (sprintf('L%d',M.(sprintf('set%d', i-1))(1,j))). ...
                        (sprintf('A%d',branch.(sprintf('es%d', i-1))(M.(sprintf('set%d', i-1))(1,j),j)));
                    for  k=1:s(3)
                        [value,position]=min([Nj, ...
                            N.(sprintf('step%d',LastNormStep(i))). ...
                            (sprintf('L%d', i)).(sprintf('A%d',branch.(sprintf('es%d',i))(i,(j-1)*s(3)+k)))]);
                        % acting in this way we obtain the first minimal value,
                        % that it is good in case of two minimal values
                        % exactly equal for two different lengths
                        if position==1 % i.e. the old estimate is still the minimal one
                            M.(sprintf('set%d', i))=[M.(sprintf('set%d', i)), ...
                                M.(sprintf('set%d', i-1))(1,j)];
                        else
                            M.(sprintf('set%d', i))=[M.(sprintf('set%d', i)),i];
                        end
                        minimalNorms_i=[minimalNorms_i,value];
                    end
                end
            end

            activeBranches=size(branch.(sprintf('es%d',i)));
            branchesAlive(i)=activeBranches(2);
        end

        %                 Estimate of beta at step i                 %


        [MaxBeta_i,posMaxBeta_i]=max(minimalNorms_i);
        normF_i(i)=MaxBeta_i*oldAlfa;
        sequenceNormF_i{i}=word.(sprintf('s%d', i))(:,posMaxBeta_i);
        [beta(i),posminmaxbeta]=min([beta(i-1),MaxBeta_i]);
        if posminmaxbeta==2
            posBeta(i)=posMaxBeta_i;
        else
            posBeta(i)=0;
        end

        %*********************************************************************
        % NB If posBeta(i)==0 it means that we are still considering the
        % previous estimate, otherwise is P.(['L' int2str(M. ...
        % (sprintf('set%d', i))(posBeta(i)))]).(['A' int2str(branch. ...
        % (sprintf('es%d',i))(M.(sprintf('set%d', i))(posBeta(i)),...
        % posBeta(i)))]) % the product with maximal rectangular norm.
        % 'M.(sprintf('set%d', i))(posBeta(i))' is its length.
        % 'posBeta(i)' is the column in 'branch.(sprintf('es%d',i))'
        % that contain, in row 'M.(sprintf('set%d', i))(posBeta(i))',
        % the position inside 'P.(['L' int2str(M.(sprintf('set%d', i))...
        % (posBeta(i)))])' % of this product
        %*********************************************************************

        % at every step we check if alfa and beta are close enough
        if abs(alfa(i)-beta(i))<=precisionJSR
            FinalText='\n    Precision required attained';
            diagnosticJSR.code=1;
            break;
        end

        if askQuestion(2)==1
            if alfa(i)>=10^-12
                relativeError=(beta(i)-alfa(i))/alfa(i);
            else
                relativeError=(beta(i)-alfa(i))/beta(i);
            end
            fprintf('\n Joint Spectral radius untill now is among {%4.15f, %4.15f}\n Relative error is around %4.15f \n',...
                alfa(i)*oldAlfa,beta(i)*oldAlfa,relativeError);
            toc
        end


        %                   i<StartingStep Section                   %


    else% i.e. i<StartingStep
        % Now we store the norm of all products of length 'i'
        LastNormStep(i)=LastNormStep(i-1);
        field_i=fieldnames(P.(sprintf('L%d', i)));
        numProd_i=size(field_i);
        NormUsingNormaStep_i=[];
        for jj = 1:numProd_i(1)
            NameMatrix_jj=char(field_i(jj));
            NormUsingNormaStep_i=[NormUsingNormaStep_i, ...
                norm(P.(sprintf('L%d', i)).(NameMatrix_jj),norma)^(1/i)];
            N.step1.(sprintf('L%d', i)).(NameMatrix_jj)= ...
                NormUsingNormaStep_i(jj);
        end
        [MaxBeta_i,posMaxBeta_i]=max(NormUsingNormaStep_i);
        normF_i(i)=MaxBeta_i*oldAlfa;
        sequenceNormF_i{i}=word.(sprintf('s%d', i))(:,posMaxBeta_i);
        [beta(i),posminmaxbeta]=min([beta(i-1),MaxBeta_i]);
        if posminmaxbeta==2
            posBeta(i)=posMaxBeta_i;
        else
            posBeta(i)=0;
        end

        %****************************************************************
        % NB If posBeta(i)==0 it means that we are still considering the
        % previous estimate, otherwise is P.(sprintf('L%d', i)). ...
        % (NameMatrix_jj) % the product with maximal rectangular norm.
        % With:   NameMatrix_jj=char(field_i(posBeta(i)));
        % and     field_i=fieldnames(P.((sprintf('L%d', i))));
        %****************************************************************

        if abs(alfa(i)-beta(i))<=precisionJSR
            % at every step we check if alfa and beta are close enough
            FinalText='\n    Precision required attained';
            diagnosticJSR.code=1;
            break;
        end
        activeBranches=size(branch.(sprintf('es%d',i)));
        branchesAlive(i)=activeBranches(2);
    end

    if askQuestion(2)==1
        fprintf(['\n' starText '\n\n   %1.0f of %1.0f active branches at step %3.0f\n\n' starText '\n'],...
            activeBranches(2),s(3)^i,i);
    end
    time(i)=toc;
end

JSRad=[alfa(length(alfa))*oldAlfa,beta(length(beta))*oldAlfa];
fprintf([barraText '\n' FinalText '\n' NormUsedText '\n\n'...
    ' Joint Spectral radius among {%4.15f, %4.15f}\n'], JSRad(1),JSRad(2));
if UseNorm~='G' && i>=StartingStep
    fprintf('    Semidefinite Programming worked %1.0f times of %1.0f total\n',...
        norm(SDPworked,1),(i-StartingStep+1));
end
fprintf([barraText '\n\n']);


% checkings
field_N=[];
numfield_N=[];
field_N=fieldnames(N);
numfield_N=size(field_N);

if norm(SDPworked,1)~=numfield_N(1)-1 && UseNorm~='G'
    warning('Something wrong in N global variable')
    diagnosticJSR.text='Something wrong in N global variable';
    return
end

if length(time)<i
    time(i)=toc;
end

%%                De-scaling of the family                %

if oldAlfa~=1
    precisionJSR=precisionJSR*oldAlfa;
    DescalingJSR_v08(i)
end

%%          Cleaning and fixing of output variables       %

if length(branchesAlive)>i
    branchesAlive(i+1:end)=[];
    rhoF_i(i+1:end)=[];
    sequenceRhoF_i(i+1:end)=[];
    normF_i(i+1:end)=[];
    sequenceNormF_i(i+1:end)=[];
    if  UseNorm=='R'
        z(:,i+1:end)=[];
    elseif UseNorm=='E'
        Z(:,:,i+1:end)=[];
    end    
end

if diagnosticJSR.code<0
    diagnosticJSR.code=0;
end
diagnosticJSR.text=[FinalText NormUsedText];
diagnosticJSR.time=time;
diagnosticJSR.norma=norma;
diagnosticJSR.FinalStep=i;
diagnosticJSR.branchesAlive=branchesAlive;
diagnosticJSR.rhoF_i=rhoF_i;
diagnosticJSR.sequenceRhoF_i=sequenceRhoF_i;
diagnosticJSR.alfa=alfa;
diagnosticJSR.beta=beta;
diagnosticJSR.posAlfa=posAlfa;
diagnosticJSR.posBeta=posBeta;
diagnosticJSR.normF_i=normF_i;
diagnosticJSR.sequenceNormF_i=sequenceNormF_i;
diagnosticJSR.z=z;
diagnosticJSR.Z=Z;
diagnosticJSR.SDPworked=SDPworked;
findposAlfa=find(posAlfa>0);
sequenceJSR=sequenceRhoF_i{findposAlfa(length(findposAlfa))}';

%%     Workspace cleaning     %


if cleanWorkspace==1
    clear global P branch dry cutted word alfa beta N M Pcut z Z ...
        posAlfa posBeta right left SDPworked oldAlfa ...
        LastNormStep;
end

end